--[[************************************************************************************************************************************************************
	
	PolaninSeeder
	
	18.07.2018 © Saty MODS (SatyMODS / http://forum.lsczech.cz)
	
****************************************************************************************************************************************************************
	
	(EN) INFO
	
	Free for use on mods. Modifications only with my permission !
	
****************************************************************************************************************************************************************
	
	(CZ) INFO
	
	Pro volne pouziti v modech. ZAKAZ modifikaci bez meho souhlasu !
	
************************************************************************************************************************************************************]]--

source(g_currentModDirectory .. "SCR/overrideScripts.lua");
source(g_currentModDirectory .. "SCR/AttachersSetStateEvent.lua");
source(g_currentModDirectory .. "SCR/WheelsSetStateEvent.lua");

PolaninSeeder = {};

function PolaninSeeder.prerequisitesPresent(specializations)
	return true;
end;

function PolaninSeeder:load(savegame)
	self.setAttachersState = SpecializationUtil.callSpecializationsFunction("setAttachersState");
	self.setWheelsState    = SpecializationUtil.callSpecializationsFunction("setWheelsState");
	
	self.getIsPlayerInRange = PolaninSeeder.getIsPlayerInRange;
		
	self.foldAttachers = false;
	self.foldWheels = false;
	self.foldAttachersNodeInRange = false;
	
	self.transportAnimation = getXMLString(self.xmlFile, "vehicle.PolaninSeeder.animations.transportAttacher#name");
	self.workAnimation = getXMLString(self.xmlFile, "vehicle.PolaninSeeder.animations.workAttacher#name");
	self.transportWheelAnimation = getXMLString(self.xmlFile, "vehicle.PolaninSeeder.animations.transportWheel#name");
	self.supportMoveAnimation = getXMLString(self.xmlFile, "vehicle.PolaninSeeder.animations.support#name");
	self.brakeColMoveAnimation = getXMLString(self.xmlFile, "vehicle.PolaninSeeder.animations.brakeCol#name");
	
	self.transportRangeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.transportAttacher#index"));
	self.transportRangeDistance = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.transportAttacher#distance"), 1);
	self.workRangeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.workAttacher#index"));
	self.workRangeDistance = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.workAttacher#distance"), 1);
	self.coverRangeIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.cover#index"));
	self.coverRangeDistance = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.PolaninSeeder.checkInRange.cover#distance"), 1);
	
	self.foldDisableAttacher = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.ridgeMarkers#foldDisableAttacher"), 0);
	
    if savegame ~= nil then
		self:setAttachersState(Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. ".PolaninSeeder#foldAttachers"), false));
		self:setWheelsState(Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key .. ".PolaninSeeder#foldWheels"), false));
	else
		self:setAttachersState(false);
	end;
end;

function PolaninSeeder:postLoad(savegame)
	self.canFoldRidgeMarker	= Utils.overwrittenFunction(RidgeMarker.canFoldRidgeMarker, PolaninSeeder.canFoldRidgeMarker);
end;

function PolaninSeeder:canFoldRidgeMarker(superFunc, state)
    if superFunc ~= nil and self.foldDisableAttacher ~= self.inputAttacherJointDescIndex and self.attacherVehicle ~= nil then
        return superFunc(self, state);
    end;
	
	return false
end;

function PolaninSeeder:getSaveAttributesAndNodes(nodeIdent)
	local nodes = nodeIdent .. '<PolaninSeeder foldAttachers="' .. tostring(self.foldAttachers) .. '" foldWheels="' .. tostring(self.foldWheels) .. '" />';
	return nil, nodes;
end;

function PolaninSeeder:update(dt)
	if self.isServer then 
		if (self.foldDisableAttacher == self.inputAttacherJointDescIndex or self.attacherVehicle == nil) and self.ridgeMarkerState ~= 0 then
			self:setRidgeMarkerState(0);
		end;
	end;

	if self.isClient then
		-- ai vehicle
		self.aiVehicleDisabled = ((self.foldAttachers and not self.foldWheels) or not self.foldAttachers);
	
		-- cover
		if self:getIsPlayerInRange(self.coverRangeIndex, self.coverRangeDistance) then
			if self.coverAnimation ~= nil then
				if self:getIsToggleCoverAllowed() then
					if self.isCoverOpen then
						g_currentMission:addHelpButtonText(g_i18n:getText("action_closeCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
					else
						g_currentMission:addHelpButtonText(g_i18n:getText("action_openCover"), InputBinding.TOGGLE_COVER, nil, GS_PRIO_NORMAL);
					end;
					
					if InputBinding.hasEvent(InputBinding.TOGGLE_COVER) then
						self:setCoverState(not self.isCoverOpen);
					end;
				end;
			end;
		end;
			
		-- fold attachers
		if self:getIsPlayerInRange(self.transportRangeIndex, self.transportRangeDistance) or self:getIsPlayerInRange(self.workRangeIndex, self.workRangeDistance) then
			if self.attacherVehicle == nil then
				if not self.foldWheels then 
					if InputBinding.hasEvent(InputBinding.PolaninSeeder_attachersOperate) then
						self:setAttachersState(not self.foldAttachers);
					end;
					if self.foldAttachers then
						g_currentMission:addHelpButtonText(g_i18n:getText("PolaninSeeder_FOLD"), InputBinding.PolaninSeeder_attachersOperate);
					else
						g_currentMission:addHelpButtonText(g_i18n:getText("PolaninSeeder_UNFOLD"), InputBinding.PolaninSeeder_attachersOperate);
					end;
				else
					if InputBinding.hasEvent(InputBinding.PolaninSeeder_attachersOperate) then
						g_currentMission:showBlinkingWarning(g_i18n:getText("PolaninSeeder_WARNING2"));
					end;
				end;
			else
				if InputBinding.hasEvent(InputBinding.PolaninSeeder_attachersOperate) then
					g_currentMission:showBlinkingWarning(g_i18n:getText("PolaninSeeder_WARNING1"));
				end;
			end;
		end;
	
		if self:getIsActiveForInput() and self.foldAttachers then
			if InputBinding.hasEvent(InputBinding.PolaninSeeder_wheelsOperate) then
				self:setWheelsState(not self.foldWheels);
			end;
			if self.foldWheels then
				g_currentMission:addHelpButtonText(g_i18n:getText("PolaninSeeder_WHEEL_UNFOLD"), InputBinding.PolaninSeeder_wheelsOperate);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("PolaninSeeder_WHEEL_FOLD"), InputBinding.PolaninSeeder_wheelsOperate);
			end;
		end;
	end;
end;

function PolaninSeeder:getIsPlayerInRange(index, minDistance)
	if g_currentMission.player ~= nil then
		local rx, ry, rz = getWorldTranslation(g_currentMission.player.rootNode);
		local ox, oy, oz = getWorldTranslation(index);
		local distance = Utils.vector3Length(rx-ox, ry-oy, rz-oz);

		if distance <= minDistance then
			return true;
		end;
	end;
		
	return false;
end;

function PolaninSeeder:onAttach(attacherVehicle)
	self.attacherVehicle = attacherVehicle;
	if self.foldAttachers then
		if self.foldWheels then
			if self.supportMoveAnimation ~= nil and self.playAnimation ~= nil then
				self:playAnimation(self.supportMoveAnimation, -1, nil, true);
			end;
		end;
	end;
	if not self.foldWheels then
		if self.brakeColMoveAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.brakeColMoveAnimation, -1, nil, true);
		end;
	end;
end;

function PolaninSeeder:onDetach()
	self.attacherVehicle = nil;
	if self.foldAttachers and self.foldWheels then
		if self.supportMoveAnimation ~= nil and self.playAnimation ~= nil  then
			self:playAnimation(self.supportMoveAnimation, 1, nil, true);
		end;
	end;
	if not self.foldWheels then
		if self.brakeColMoveAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.brakeColMoveAnimation, 1, nil, true);
		end;
	end;
end;

function PolaninSeeder:setAttachersState(state, noEventSend)
	AttachersSetStateEvent.sendEvent(self, state, noEventSend);
	self.foldAttachers = state;
	
	if self.foldAttachers then
		if self.workAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.workAnimation, -1, nil, true);
		end;
		if self.transportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportAnimation, 1, nil, true);
		end;
	else
		if self.transportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportAnimation, -1, nil, true);
		end;
		if self.workAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.workAnimation, 1, nil, true);
		end;
	end;
end;

function PolaninSeeder:setWheelsState(state, noEventSend)
	WheelsSetStateEvent.sendEvent(self, state, noEventSend);
	self.foldWheels = state;
	
	if self.foldWheels then
		if self.transportWheelAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportWheelAnimation, 1, nil, true);
		end;
	else
		if self.transportWheelAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportWheelAnimation, -1, nil, true);
		end;
	end;
end;

function PolaninSeeder:readStream(streamId, connection)
	self:setAttachersState(streamReadBool(streamId), true);
	self:setWheelsState(streamReadBool(streamId), true);
end;

function PolaninSeeder:writeStream(streamId, connection)
	streamWriteBool(streamId, self.foldAttachers);
	streamWriteBool(streamId, self.foldWheels);
end;

function PolaninSeeder:delete() end;

function PolaninSeeder:draw()
    if self.isClient then
        if not self.foldAttachers then
            g_currentMission:addExtraPrintText(g_i18n:getText("PolaninSeeder_HELP1"));
        end;
        if self.foldAttachers and not self.foldWheels then
            g_currentMission:addExtraPrintText(g_i18n:getText("PolaninSeeder_HELP2"));
        end;
    end
end;

function PolaninSeeder:updateTick(dt) end;

function PolaninSeeder:mouseEvent(posX, posY, isDown, isUp, button) end;

function PolaninSeeder:keyEvent(unicode, sym, modifier, isDown) end;


local AIVehicleCanStartAIVehicle = AIVehicle.canStartAIVehicle
function AIVehicle:canStartAIVehicle()
	for _, implement in pairs(self.attachedImplements) do
		if implement.object ~= nil then
			if implement.object.aiVehicleDisabled then
				return false;
			end;
		end;
	end;
	
	return AIVehicleCanStartAIVehicle(self);
end;